#!/usr/bin/ruby
# encoding: UTF-8
require 'rubygems'
require 'getoptlong'
require 'fileutils'
require 'date'
#---------------------------------------------------------------------------------------------------
# Dr. Heinz Breinlinger
#
# Verarbeitet eine Suchanfrage innerhalb der Programmdatei und gibt das Ergebnis als 3 HTML-Dateien,
# sortiert nach Anfangsdatum/-zeit, Titel und Kanal aus.
#---------------------------------------------------------------------------------------------------

#{{{{Globale Variablen
$mapDay={0 => "So",
         1 => "Mo",
         2 => "Di",
         3 => "Mi",
         4 => "Do",
         5 => "Fr",
         6 => "Sa"}
#------------------------------------------------------------------------------
# Hash der Kommentarfelder des Fernsehprogrammes
# $tvDescriptionHash={ "#327" => "Es war einmal ..."}
#------------------------------------------------------------------------------
$tvDescriptionHash=Hash.new
$htmlHeader="Kriterien:"
#}}}}
#{{{{die
def die(message)
  puts("#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!")
  puts("Error: #{message}")
  puts("#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!")
  exit 1
end
#}}}}
#{{{{episodeMatchesSearchTime
def episodeMatchesSearchTime(start,finish)
  #-------------------------------------------------------------------------------
  # start, finish sind zunaechst Strings (2013-10-16 14:25:00) und werden spaeter
  # zu Time-Objekten
  #-------------------------------------------------------------------------------
  begin
    if(($searchTime.class.to_s != "Time"))
      die("episodeMatchesSearchTime: $searchTime ist kein Time-Objekt #{$searchTime.class}")
    end
    [start,finish].each{ |timestamp|
      if(! timestamp.match(/^\d{4}-\d{2}-\d{2}\s+\d{2}:\d{2}:\d{2}$/))
        die("episodeMatchesSearchTime: Falsches Format #{timestamp}")
      end
    }
    start=DateTime.parse(start).to_time.gmtime
    #-----------------------------------------------------------------------------
    # Anfangszeit vor oder identisch mit Suchzeit ?
    #-----------------------------------------------------------------------------
    if(isNotAfter($searchTime,start))
      finish=DateTime.parse(finish).to_time.gmtime
      #---------------------------------------------------------------------------
      # Endzeit nach Suchzeit ?
      #---------------------------------------------------------------------------
      if(isNotAfter($searchTime,finish))
        return false
      else
        return true
      end
    else
      return false
    end
  rescue => exception
    die("episodeMatchesSearchTime: #{exception.message}")
  end
end
#}}}}
#{{{{getToday
def getToday()
  now=Time.now()
  return "#{now.year}-#{pad(now.month)}-#{pad(now.day)}"
end
#}}}}
#{{{{getWorkDir
def getWorkDir()
  #------------------------------------------------------------------------------
  # epgData-Verzeichnis aus Konfiguration ermitteln
  #------------------------------------------------------------------------------
  config="/epg/config"
  $workDir=""
  unless(File.exists?(config))
    die("Datei #{config} existiert nicht")
  end
  begin
    File.open(config,"r"){ |file|
      file.each_line{ |line|
        if(line.match(/workDir=(\S+)$/))
          $workDir=$1.to_s
          unless(Dir.exists?($workDir))
            die("Verzeichnis #{$workDir} existiert nicht")
          end
          break
        end
      }
    }
  rescue => exception
    die("#{exception.message}")
  end
  if($workDir.empty?)
    die("Konnte $workDir nicht aus #{config} extrahieren")
  end
end
#}}}}
#{{{{isNotAfter
def isNotAfter(time1,time2)
  #------------------------------------------------------------------
  # time1 ist der Benchmark, time2 die zu pruefende Zeit
  #------------------------------------------------------------------
  begin
    [time1,time2].each{ |timestamp|
      if(timestamp.class.to_s != "Time")
        die("isNotAfter: Ungueltiges Objekt #{timestamp.class}")
      end
    }
    if((time2-time1) > 0)
      return false
    else
      return true
    end
  rescue => exception
    die("isNotAfter: #{exception.message}")
  end
end
#}}}}
#{{{{pad
def pad(value)
  if (value < 10)
    return String.new("0#{value}")
  else
    return value
  end
end
#}}}}
#{{{{printDescriptionsHtml
def printDescriptionsHtml(file)
  file=File.open(file,"w")
  #--------------------------------------------------------------------------------------------------
  file.puts('<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">')
  file.puts('<html>')
  file.puts('<head>')
  file.puts('<title>Beschreibungen</title>')
  #----------------------------------------
  file.puts('<style type="text/css">')
  file.puts('  @import url(/master.css);')
  file.puts(' </style>')
  #----------------------------------------
  file.puts('</head>')
  file.puts('<body>')
  file.puts('<span style="font-family:Arial">')
  #--------------------------------------------------------------------------------------------------
  file.puts('<table border="1" class="tv">')
  $tvDescriptionHash.each{ |epId,description|
    epIdShort=epId.sub(/#/,"")
    file.puts("<tr>")
    file.puts("<td><a name=\"#{epIdShort}\"></a>#{description}</td>")
    file.puts("</tr>")
  }
  file.puts('</table>')
  #--------------------------------------------------------------------------------------------------
  file.puts("</span>")
  file.puts("</body>")
  file.puts("</html>")
  file.close()
end
#}}}}
#{{{{printEpisodeHtml
def printEpisodeHtml(episodeArr,targetFile,file)
  file.puts("<tr>")
  #-----------------------------------------------------------------------------------------------
  epId=episodeArr[0].match(/\#\d+/).to_s
  epIdShort=epId.sub(/#/,"")
  epChannel=episodeArr[0].match(/^[[:alnum:]]+/)
  epWeekDay=$mapDay[DateTime.parse(episodeArr[1]).wday]
  epDate=episodeArr[1].match(/\d{4}-(\d{2})-(\d{2})/)
  epMonth=$1; epDay=$2; epDate=String.new("#{epDay}.#{epMonth}")
  epStart=episodeArr[1].match(/\d{2}:\d{2}/)
  epEnd=episodeArr[2].match(/\d{2}:\d{2}/)
  epCategory=episodeArr[4].chomp
  epTitle=episodeArr[5].chomp
  epSubtitle=episodeArr[6].chomp
  epGenre=episodeArr[8].chomp
  epPic=episodeArr[9].match(/[[:print:]]+\.jpg/).to_s # ist leer, wenn kein Bild vorhanden
  #-----------------------------------------------------------------------------------------------
  file.puts("<td>#{epDate}</td>")
  file.puts("<td>#{epWeekDay}</td>")
  file.puts("<td>#{epStart}</td>")
  file.puts("<td>#{epEnd}</td>")
  file.puts("<td>#{epTitle}</td>")
  file.puts("<td>#{epSubtitle}</td>")
  file.puts("<td>#{epChannel}</td>")
  if($tvDescriptionHash.has_key?(epId))
    file.puts("<td><a href=\""+targetFile+"##{epIdShort}\" target=\"rechtsoben\">Text</a></td>")
  else
    file.puts("<td></td>")
  end
  if(! epPic.empty?)
    file.puts("<td><a href=\""+epPic+"\" target=\"rechtsunten\">Bild</a></td>")
  else
    file.puts("<td></td>")
  end
  file.puts("<td>#{epCategory}</td>")
  file.puts("<td>#{epGenre}</td>")
  file.puts("<td><input type=\"checkbox\" name=\"episode[]\" id=\"#{epIdShort}\" value=\"#{epIdShort}\"></td>")
  #-----------------------------------------------------------------------------------------------
  file.puts("</tr>")
end
#}}}}
#{{{{printFrame
def printFrame(sourceFile,targetFile,file)
  file=File.open(file,"w")
  file.puts('<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Frameset//EN" "http://www.w3.org/TR/html4/frameset.dtd">')
  file.puts('<html>')
  file.puts('<head>')
  file.puts('<title>Frame</title>')
  file.puts('</head>')
  file.puts('<frameset cols="60%,40%">')
    file.puts('<frame src="'+sourceFile+'" scrolling="yes" name="links">')
    file.puts('<frameset rows="50%,50%">')
      file.puts('<frame src="'+targetFile+'" name="rechtsoben">')
      file.puts('<frame src="" name="rechtsunten">')
    file.puts('</frameset>')
  file.puts('</frameset>')
  file.puts('</html>')
  file.close()
end
#}}}}
#{{{{printTVHtml
def printTVHtml(hash,targetFile,file)
  file=File.open(file,"w")
  #--------------------------------------------------------------------------------------------------
  file.puts('<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">')
  file.puts('<html>')
  file.puts('<head>')
  file.puts('<title>TV-Programm</title>')
  file.puts('<style type="text/css">')
  file.puts('  @import url(/master.css);')
  file.puts(' </style>')
  file.puts('</head>')
  file.puts('<body>')
  file.puts('<span style="font-family:Arial">')
  file.puts("<em class=\"file\">#{$htmlHeader}</em>")
  #--------------------------------------------------------------------------------------------------
  file.puts('<form action="record.php" method="post">')
  file.puts('<p><input type="submit" name="submitButton" value="Aufzeichnen"> <input type="reset" name="resetButton" value="Zur&uuml;cksetzen"></p>')
  #--------------------------------------------------------------------------------------------------
  file.puts('<table border="1" class="tv"')
  hash.sort.each{ |key,episodeArr|
    printEpisodeHtml(episodeArr,targetFile,file)
  }
  file.puts('</table>')
  #--------------------------------------------------------------------------------------------------
  file.puts('</form>')
  #--------------------------------------------------------------------------------------------------
  file.puts("</span>")
  file.puts("</body>")
  file.puts("</html>")
  file.close()
end
#}}}}
#{{{{Variablen
usage="mytubehtml.rb -[tsTcCdg] <pattern> 
  -p(refix) <prefix>  => <prefix>_FrameTime.htm, <prefix>_TitleFrame.htm, <prefix>_ChannelFrame.htm
  -t(itle)  
  -C(ategory)
  -g(enre)
  -s(ubTitle) 
  -T(ime)   \"2013-11-16 23:30:00\" (Sendung laeuft um diese Zeit)
            \"2013-11-16\"          (Sendung beginnt an diesem Tag)
  -c(hannel) \"(arte|phoenix|3sat)\"
  -d(escription)"

searchTitle=".*"
searchSubTitle=".*"
$searchTime=".*"
$searchATime=false
$searchADay=false
searchChannel=".*"
searchDescription=".*"
searchGenre=".*"
searchCategory=".*"
prefix=""
timeFile="Zeit.htm"
frameFile="Programm.htm"
descriptionFile="Beschreibung.htm"
#------------------------------------------------------------------------------
# Hashes mit verschiedenen Key-Kriterien fuer ein und dasselbe Fernsehprogramm
#
# tvTimeHash={ "2011-12-10 20:15:00" => [episodeArr0,episodeArr1,....],
#           "2011-12-10 21:30:00" => [episodeArr15,episodeArr28,...]
#         }
# tvTitleHash={ "Zdf-History" => [episodeArr4,...]}
# tvChannelHash={ "3sat" => [episodeArr7,...]}
#------------------------------------------------------------------------------
tvTimeHash=Hash.new
tvTitleHash=Hash.new
tvChannelHash=Hash.new
#}}}}
#{{{{Kommandozeilenoptionen
options=GetoptLong.new(
  ['--help', '-h', GetoptLong::NO_ARGUMENT],
  ['--title', '-t', GetoptLong::REQUIRED_ARGUMENT],
  ['--prefix', '-p', GetoptLong::REQUIRED_ARGUMENT],
  ['--subtitle', '-s', GetoptLong::REQUIRED_ARGUMENT],
  ['--genre', '-g', GetoptLong::REQUIRED_ARGUMENT],
  ['--category', '-C', GetoptLong::REQUIRED_ARGUMENT],
  ['--Time', '-T', GetoptLong::REQUIRED_ARGUMENT],
  ['--channel', '-c', GetoptLong::REQUIRED_ARGUMENT],
  ['--description', '-d', GetoptLong::REQUIRED_ARGUMENT],
)
options.each{ |opt,arg|
  case opt
    when '--help'
      puts("#{usage}")
      exit
    when '--title'
      searchTitle=arg
    when '--subtitle'
      searchSubTitle=arg
    when '--Time'
      unless(arg == ".*")
        if(arg.match(/^\d{4}-\d{2}-\d{2}$/))
          $searchTime=arg
          $searchADay=true
        elsif(arg.match(/^\d{4}-\d{2}-\d{2}\s+\d{2}:\d{2}:\d{2}$/))
          $searchTime=DateTime.parse(arg).to_time.gmtime
          $searchATime=true
        else
          die("Zeitformat ungueltig: #{arg} Soll: 2013-11-16[ 22:30:00]")
        end
      end
    when '--channel'
      searchChannel=arg
    when '--description'
      searchDescription=arg
    when '--genre'
      searchGenre=arg
    when '--category'
      searchCategory=arg
    when '--prefix'
      prefix=String.new(arg+"_")
      timeFile="#{prefix}#{timeFile}"
      frameFile="#{prefix}#{frameFile}"
      descriptionFile="#{prefix}#{descriptionFile}"
  end
}
#--------------------------------------------------------------------------
getWorkDir
$htmlDir=$workDir+"/Html"
unless(Dir.exists?($htmlDir))
  die("Verzeichnis #{$htmlDir} existiert nicht")
end
qualProgrammeFile="#{$workDir}/programme"
qualTimeFile="#{$htmlDir}/#{timeFile}"
qualFrameFile="#{$htmlDir}/#{frameFile}"
qualDescriptionFile="#{$htmlDir}/#{descriptionFile}"
$htmlHeader="Kriterien: #{searchTitle}_#{searchSubTitle}_#{$searchTime}_#{searchChannel}_#{searchDescription}_#{searchGenre}_#{searchCategory}"
#}}}}
#{{{{Hauptverarbeitung
#------------------------------------------------------------------------------
# Hauptverarbeitung
#--------------------------------------------------------------------------------
unless(File.exists?(qualProgrammeFile))
  die("Kann Datei #{qualProgrammeFile} nicht finden")
end
file=File.open(qualProgrammeFile, :encoding => "UTF-8")
#--------------------------------------------------------------------------------
while(line=file.gets) do                                      
  if(line.match(/<d2>#{searchChannel} /i))
    episodeArr=Array.new
    episodeArr.push(line)                          #episodeArr[0] = <d2> (Kanal und Sendungs-ID)
    episodeArr.push(line=file.gets)                #episodeArr[1] = <d4> (Datum und Uhrzeit Beginn)
    if($searchADay) 
      if(line.match(/#{$searchTime}/i))
        episodeArr.push(line=file.gets)            #episodeArr[2]  = <d5> (Endzeit)
      else
        next # Interation
      end
    elsif($searchATime) 
      line.match(/<d4>(\d{4}-\d{2}-\d{2}\s+\d{2}:\d{2}:\d{2})/i)
      start=$1.to_s
      (start.empty?) && die("Konnte Anfangszeit nicht aus #{line} extrahieren")
      episodeArr.push(line=file.gets)              #episodeArr[2]  = <d5> (Endzeit)
      line.match(/<d5>(\d{4}-\d{2}-\d{2}\s+\d{2}:\d{2}:\d{2})/i)
      finish=$1.to_s
      (finish.empty?) && die("Konnte Endzeit nicht aus #{line} extrahieren")
      unless(episodeMatchesSearchTime(start,finish))
        next # Iteration
      end
    #----------------------------------------
    # beliebige Zeit
    #----------------------------------------
    else
      episodeArr.push(line=file.gets)              #episodeArr[2]  = <d5> (Endzeit)
    end 
    episodeArr.push(line=file.gets)                #episodeArr[3] = <d7> (Dauer)
    episodeArr.push(line=file.gets)                #episodeArr[4] = <d10> (Kategorie)
    if(line.match(/<d10>.*#{searchCategory}.*/i))   
      episodeArr.push(line=file.gets)              #episodeArr[5] = <d19> (Titel)
      if(line.match(/<d19>.*#{searchTitle}.*/i))
        episodeArr.push(line=file.gets)            #episodeArr[6]   = <d20> (Untertitel)
        if(line.match(/<d20>.*#{searchSubTitle}.*/i))
          episodeArr.push(line=file.gets)          #episodeArr[7]   = <d21> (Beschreibung)
          if(line.match(/<d21>.*#{searchDescription}/i))
            episodeArr.push(line=file.gets)        #episodeArr[8]   = <d25> (Genre)
            if(line.match(/<d25>.*#{searchGenre}/i))
              episodeArr.push(file.gets)           #episodeArr[9]   = <d40> (Bild)
              #----------------------------------------------
              # Zeilen aufbereiten ...
              #----------------------------------------------
              episodeArr.each{ |line|
                line.gsub!(/<\/*d\d+>/,"")
                line.gsub!(/^[[:blank:]]+/,"")
                line.gsub!(/[[:blank:]]+$/,"")
                line.gsub!(/ä/,"ae")
                line.gsub!(/ö/,"oe")
                line.gsub!(/ü/,"ue")
                line.gsub!(/Ä/,"Ae")
                line.gsub!(/Ö/,"Oe")
                line.gsub!(/Ü/,"Ue")
                line.gsub!(/ß/,"ss")
                line.sub!(/\n/,"")
              }
              epId=episodeArr[0].match(/\#\d+/).to_s
              epChannel=episodeArr[0].match(/^[[:alnum:]]+/).to_s
              epStart=episodeArr[1].chomp
              epTitle=episodeArr[5].chomp
              key=String.new(epStart+"_"+epTitle+"_"+epChannel)
              tvTimeHash[key]=episodeArr
              key=String.new(epTitle+"_"+epStart+"_"+epChannel)
              tvTitleHash[key]=episodeArr
              key=String.new(epChannel+"_"+epStart+"_"+epTitle)
              tvChannelHash[key]=episodeArr
              unless(episodeArr[7].empty?)
                $tvDescriptionHash[epId]=episodeArr[7]
              end
            end
          end
        end
      end
    end
  end
end
file.close()
printTVHtml(tvTimeHash,descriptionFile,qualTimeFile)
printDescriptionsHtml(qualDescriptionFile)
printFrame(timeFile,descriptionFile,qualFrameFile)
#}}}}
